%****************   Controle de Processos - Pndulo Invertido   
****************

%**** Projeto de um controlador digital para o sistema de pndulo invertido 
com carro

%**** Este  o programa para calcular KK (a matriz K cincunflexo (K^) ).
% Esta  uma matriz 1x5 onde as primeiras quatro colunas so de ganho de 
retroao de estado
% e a quinta coluna  a constante de ganho integral. A frmula de Ackermann 
 utilizada na
% resoluo para a matriz KK

%**** Entrada das variaveis
A=[ 0 1 0 0 ; 20.601 0 0 0 ; 0 0 0 1 ; -0.4905 0 0 0 ]
B=[ 0 ; -1 ; 0 ; 0.5 ]
C=[ 0 0 1 0 ]
D=0

%**** Discretizar a equacao de estado
[G,H]=c2d( A , B , 0.1 )

%**** Entrar com as matrizes
G1=[ G zeros(4,1) ; -C*G 1 ]
H1=[ H ; -C*H]

%**** Entrar com a matriz de controlabilidade M
M=[ H1 G1*H1 G1^2*H1 G1^3*H1 G1^4*H1 ]

%**** Verificar o posto da matriz
rank(M)

%**** Como o posto de M  5, a alocao arbitrria de plos  possvel

%**** Determinar o polinmio caracterstico desejado,
%     entrando com a matriz J e calculando poly(J)

J=[ 0.9+0.25*i 0 0 0 0 ; 0 0.9-0.25*i 0 0 0 ; 0 0 0 0 0; 0 0 0 0 0 ; 0 0 0 0 
0 ]
JJ=poly(J)

%**** Entrar com o polinmio caracterstico Phi
Phi=polyvalm(poly(J),G1)

%**** A matriz KK  dada pela equao seguinte
KK=[ 0 0 0 0 1]*(inv(M))*Phi

K1=KK(1)
K2=KK(2)
K2=KK(3)
K4=KK(4)
Kl=-KK(5)

%**** Em seguida, ser obtida a resposta em degrau do sistema projetado
K=[ KK(1)  KK(2)  KK(3)  KK(4) ]
GG=[ G-H*K H*Kl ; -C*G+C*H*K 1-C*H*Kl ]
HH=[ 0 ; 0 ; 0 ; 0 ; 1 ]
CC=[ C 0 ]
DD=[ 0 ]

%**** Para obter a resposta em degrau unitrio, deve-se primeiro transformar 
as equaes
%     no espao de estados na funo de transferncia discreta
[num,den]=ss2tf(GG,HH,CC,DD)

%**** Entrada das variaveis
FF=[ 1 0 0 0 0 ]
LL=[ 0 0 0 1 0 ]
MM=[ 0 0 0 0 1 ]
JJ=[ 0 1 0 0 0 ]

%**** Comandos para obter a resposta em degrau unitrio
r=ones(1,101)
axis([ 0 100 -0.5 2 ])
k=0:100
y=filter(num,den,r)
%subplot(2,3,1)
plot(k,y,'bo',k,y,'r-')
grid
whitebg
title('Posio do carro y(k)=x3(k)')
xlabel('k')
ylabel('y(k)=x3(k)')

pause

%**** Para obter x1(k), converter as equaes no espao de estados na funo 
de
%     transferncia discreta X1(z)/R(z)
[num1,den1]=ss2tf(GG,HH,FF,DD)

%**** Comandos para obter a resposta em degrau unitrio
axis([ 0 100 -1 1 ])
x1=filter(num1,den1,r)
%subplot(2,3,2)
plot(k,x1,'bo',k,x1,'-')
grid
title('Deslocamento angular Teta: x1(k)')
xlabel('k')
ylabel('x1(k)')

%**** Para obter x2(k), converter as equaes no espao de estados na funo 
de
%     transferncia discreta X2(z)/R(z)
[num2,den2]=ss2tf(GG,HH,JJ,DD)

pause

%**** Comandos para obter a resposta em degrau unitrio
axis([ 0 100 -5 10 ])
x2=filter(num2,den2,r)
%subplot(2,3,3)
plot(k,x2,'bo',k,x2,'-')
grid
title('Deslocamento angular Teta: x2(k)')
xlabel('k')
ylabel('x2(k)')

pause

%**** Para obter x4(k), converter as equaes no espao de estados na funo 
de
%     transferncia discreta X4(z)/R(z)
[num4,den4]=ss2tf(GG,HH,LL,DD)

%**** Comandos para obter a resposta em degrau unitrio
axis([ 0 100 -5 5 ])
x4=filter(num4,den4,r)
%subplot(2,3,4)
plot(k,x4,'bo',k,x4,'-')
grid
title('Velocidade do Carro: x4(k)')
xlabel('k')
ylabel('x4(k)')

pause

%**** Para obter x5(k), converter as equaes no espao de estados na funo 
de
%     transferncia discreta X5(z)/R(z)
[num5,den5]=ss2tf(GG,HH,MM,DD)

%**** Comandos para obter a resposta em degrau unitrio
axis([ 0 100 -2 8 ])
x5=filter(num5,den5,r)
%subplot(2,3,5)
plot(k,x5,'bo',k,x5,'-')
grid
title('Sada do Integrador: x5(k)=v(k)')
xlabel('k')
ylabel('x5(k)=v(k)')

pause
